@@ -40,6 +40,7 @@
 static gint text_id = 0;
 static GObjectClass *parent_class;
 
+static gboolean festival_server_exists = FALSE;
 
 static gchar *
 festival_get_version (void)
@@ -68,6 +69,18 @@ festival_start (FestivalSynthesisDriver *d)
 {
 	int festival_input[2];
 	int festival_output[2];
+	gchar *s = NULL;
+	
+	if ((s = g_find_program_in_path ("festival")) != NULL)
+	{
+	    g_free (s);
+	    festival_server_exists = TRUE;
+	}
+	else
+	{
+	    festival_server_exists = FALSE;
+	    return FALSE;
+	}
 
 	/* Create pipes */
 
@@ -97,9 +110,10 @@ festival_start (FestivalSynthesisDriver *d)
 	
 	/* Set things up for async mode */
 
-	festival_synthesis_driver_say_raw (d, "(audio_mode 'async)\n");
+	if (festival_server_exists)
+	    festival_synthesis_driver_say_raw (d, "(audio_mode 'async)\n");
 
-	return TRUE;
+	return festival_server_exists;
 }
 
 
@@ -167,17 +181,20 @@ get_voice_list (void)
 	GSList *l = NULL;
 	GNOME_Speech_VoiceInfo *info;	
 
-	info = GNOME_Speech_VoiceInfo__alloc ();
-	info->language = CORBA_string_dup("en_US");
-	info->name = CORBA_string_dup ("Kevin");
-	info->gender = GNOME_Speech_gender_male;
-	l = g_slist_prepend (l, info);
-	info = GNOME_Speech_VoiceInfo__alloc ();
-	info->language = CORBA_string_dup("en_US");
-	info->name = CORBA_string_dup ("Kal");
-	info->gender = GNOME_Speech_gender_male;
-	l = g_slist_prepend (l, info);
-	l = g_slist_reverse (l);
+	if (festival_server_exists)
+	{
+	    info = GNOME_Speech_VoiceInfo__alloc ();
+	    info->language = CORBA_string_dup("en_US");
+	    info->name = CORBA_string_dup ("Kevin");
+	    info->gender = GNOME_Speech_gender_male;
+	    l = g_slist_prepend (l, info);
+	    info = GNOME_Speech_VoiceInfo__alloc ();
+	    info->language = CORBA_string_dup("en_US");
+	    info->name = CORBA_string_dup ("Kal");
+	    info->gender = GNOME_Speech_gender_male;
+	    l = g_slist_prepend (l, info);
+	    l = g_slist_reverse (l);
+	}
 	return l;
 }
 
@@ -490,6 +507,7 @@ main (int argc,
 	obj_id = "OAFIID:GNOME_Speech_SynthesisDriver_Festival:proto0.3";
 
 	driver = festival_synthesis_driver_new ();
+
 	if (!driver)
 		g_error ("Error creating speech synthesis driver object.\n");
 
