@@ -94,10 +94,11 @@ select_server (CORBA_Environment *ev)
 		return CORBA_OBJECT_NIL;
 	}
 	info = &servers->_buffer[i];
-	printf ("Atempting to activate %s.\n", info->iid);
+	printf ("Attempting to activate %s.\n", info->iid);
 	rv = bonobo_activation_activate_from_id (
 		(const Bonobo_ActivationID) info->iid,
 		0, NULL, ev);
+	if (BONOBO_EX (ev)) printf ("Error activating service: %s\n", bonobo_exception_get_text (ev));
 	CORBA_free (servers);
 
 	return rv;
@@ -236,6 +237,7 @@ do_test (GNOME_Speech_SynthesisDriver driver, CORBA_Environment *ev)
 	CORBA_string driver_name, driver_version;
 	CORBA_string synth_name, synth_version;
 	GNOME_Speech_Speaker speaker;
+	GNOME_Speech_voice_gender gender;
 	GNOME_Speech_VoiceInfoList *voices;
 	gint i;
     
@@ -264,9 +266,37 @@ do_test (GNOME_Speech_SynthesisDriver driver, CORBA_Environment *ev)
 		CORBA_free (synth_version);
 	}
 
+	input[0] = '\0';
+	while (*input != 'm' && *input != 'f') {
+		printf ("\nEnter desired gender ('m' or 'f'): ");
+		fgets (input, 80, stdin);
+	}
+
+	gender = (*input == 'f') ? GNOME_Speech_gender_female : GNOME_Speech_gender_male;
+
+	input[0] = '\0';
+	while (strlen (input) == 0) {
+		printf ("\nEnter desired locale, or 'all' to display all voices: ");
+		fgets (input, 80, stdin);
+	}
+
 	/* Display list of voices */
 
-	voices = GNOME_Speech_SynthesisDriver_getAllVoices (driver, ev);
+	if (!strcmp (input, "all\n") || strlen (input) == 1)
+	{
+	    voices = GNOME_Speech_SynthesisDriver_getAllVoices (driver, ev);
+	}
+	else
+	{
+	    GNOME_Speech_VoiceInfo *voice_spec;
+	    input [strlen (input) - 1] = '\0';
+	    voice_spec = GNOME_Speech_VoiceInfo__alloc ();
+	    voice_spec->name = CORBA_string_dup ("");
+	    voice_spec->language = CORBA_string_dup (input);
+	    voice_spec->gender = GNOME_Speech_gender_male;
+
+	    voices = GNOME_Speech_SynthesisDriver_getVoices (driver, voice_spec, ev);
+	}
 	if (BONOBO_EX (ev)) {
 		fprintf (stderr, "Exception %s getting voice list.\n",
 			 bonobo_exception_get_text (ev));
@@ -287,7 +317,7 @@ do_test (GNOME_Speech_SynthesisDriver driver, CORBA_Environment *ev)
 				 bonobo_exception_get_text (ev));
 			return -1;
 		}
-		speaker_say_print (speaker, "%d. %s\n", i+1, voices->_buffer[i].name);
+		speaker_say_print (speaker, "%d. %s (language %s)\n", i+1, voices->_buffer[i].name, voices->_buffer[i].language);
 		bonobo_object_release_unref (speaker, NULL);
 	}
 	while (choice < 1 || choice > voices->_length) {
@@ -360,17 +390,20 @@ main (int argc, char **argv)
 	o = select_server (&ev);
 	if (BONOBO_EX (&ev) || o == CORBA_OBJECT_NIL)
 	{
-		printf ("No server selected.\n");
+		fprintf (stderr, "Server selection failed: %s.\n", 
+			BONOBO_EX (&ev) ? bonobo_exception_get_text (&ev) : "object is NIL");
 		return 0;
 	}
 
-	/* Atempt to initialize the driver */
+	/* Attempt to initialize the driver */
 
 	if (GNOME_Speech_SynthesisDriver_driverInit (o, &ev))
 		while (!done) {
 			if (do_test (o, &ev))
 				done = 1;
 		}
+        else
+	        fprintf (stderr, "Server could not be initialized.\n");
 	bonobo_object_release_unref (o, NULL);
 	CORBA_exception_free (&ev);
 
